/********************************************

OpenGL program by Andreas Gustafsson (C) 2000

********************************************/

#define WIN // Use Windows timer and sound 
#define MAX_NR_SHOTS 16

#include "stdio.h"
#include "math.h"
//This program requires the OpenGL and GLUT libraries
// You can obtain them for free from http://www.opengl.org
#include "GL/glut.h"
#include "texture.h"

#ifdef WIN
#include "mmsystem.h"
#endif
double time=0,deltaTime=0;

#ifdef WIN
static LARGE_INTEGER TimeFreq, TimeStart;
int updateFPS=0;
double curFPS=0;
char titleString[100];
#endif

int haltMovement=0;
CTexture texture;
CTexture bgTexture;
GLUquadric *quadObj;

typedef struct lsr
{
	float rx;
	float ry;
	float rz;
	float px;
	float py;
	float pz;
	bool active;
} laser_shot;

laser_shot shots[16];
int nrShots=0;

void UpdateTimer()
{
	#ifdef WIN
	static double oldTime;
   LARGE_INTEGER TimeNow;
   QueryPerformanceCounter(&TimeNow);
   time=((TimeNow.QuadPart-TimeStart.QuadPart)*1000/TimeFreq.QuadPart);
   time=time/1000;	
   deltaTime=time-oldTime;
   oldTime=time;

	if(deltaTime>0) curFPS+=1.f/deltaTime;
	updateFPS++;
	if(updateFPS==100)
	{
		sprintf(titleString,"OpenGL program by Andreas Gustafsson (Shadow/Noice) (C) 2000   -=[ FPS:%.0f ]=-",curFPS/100.0);
   	glutSetWindowTitle(titleString);
   	updateFPS=0;
   	curFPS=0;
   }
	#else
   deltaTime=0.01;
   time+=deltaTime;
	#endif
}

void Resize( GLsizei iWidth, GLsizei iHeight ) 
{ 
	glViewport (0,0,iWidth,iHeight); 
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(55.0,(GLfloat)iWidth/(GLfloat)iHeight, 1.0, 20.0);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glTranslatef (0.0, 0.0, -3.0);
}

void Keyboard(unsigned char cKey, int iX, int iY)
{
   switch(cKey)
   {
		case ' ' :
      {
      	haltMovement=1-haltMovement;
   	} 
   	break;
   }
}

void Idle()
{
	glutPostRedisplay();
}

void Init()
{

	float params[4];
	texture.LoadJPG("ball.jpg");
	bgTexture.LoadJPG("space2.jpg");
	quadObj=gluNewQuadric();

	params[0]=0.75;
	params[1]=0.75;
	params[2]=0.75;
	params[3]=0;
	glLightfv(GL_LIGHT0,GL_DIFFUSE,params);
	params[0]=-0.25;
	params[1]=0.25;
	params[2]=1;
	params[3]=0;
	glLightfv(GL_LIGHT0,GL_POSITION,params);
	params[0]=0.8;
	params[1]=0.8;
	params[2]=0.8;
	params[3]=0.0;
	glMaterialfv(GL_FRONT,GL_DIFFUSE,params);
}

void DrawLaser(int nr)
{
	int nrLayers=10;
	float intens=1.5;
	
	glPushMatrix();
//	glDisable(GL_DEPTH_TEST);
	glDepthMask(0);
	glEnable(GL_BLEND);
	glBlendFunc(GL_ONE,GL_ONE);
   glColor3f(1.0/(nrLayers/intens),0.3/(nrLayers/intens),0.2/(nrLayers/intens));
   texture.Use();
   glRotatef(shots[nr].rx,1,0,0);
   glRotatef(shots[nr].ry,0,1,0);
   glRotatef(shots[nr].rz,0,0,1);
   glTranslatef(shots[nr].px,shots[nr].py,shots[nr].pz);
   glScalef(1.0,0.1,0.1);
   glEnable(GL_TEXTURE_2D);
   for(int i=0;i<nrLayers;i++)
   {
   	glBegin(GL_QUADS);
		glTexCoord2f(0,0);
		glVertex3f(-1,-1,0);
		glTexCoord2f(1,0);
		glVertex3f(1,-1,0);
		glTexCoord2f(1,1);
		glVertex3f(1,1,0);
		glTexCoord2f(0,1);
		glVertex3f(-1,1,0);
		glEnd();	
   	glRotatef(180.0/(float)(nrLayers),1,0,0);
   }	
   glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
   glPopMatrix();
	glDepthMask(1);
}

void DrawSpace()
{
	float x,y;
	
	glPushMatrix();
	glDisable(GL_BLEND);
	glRotatef(15.0f*sin(time*0.17), 1.f, 0, 0 );
	glRotatef(15.0f*sin(time*0.24), 0,1.f, 0 );
	glRotatef(time*5.0f, 0, 0, 1.f );
	glEnable( GL_TEXTURE_2D );
	glScalef(3.5,3.5,1);
	bgTexture.Use();
	glBegin(GL_QUADS);
	glColor3f(0.75f,0.75f,0.75f);
	//glColor3f(1,1,1);
	for(y=-8;y<8;y+=1)
	{
		for(x=-8;x<8;x+=1)
		{
			glTexCoord2f( (x+8)/16, (y+8)/16);
			glVertex3f(x/8,y/8,-2+sqrt((float)((x+0)*(x+0)+(y+0)*(y+0)))/4.0);
			glTexCoord2f( (x+9)/16, (y+8)/16);
			glVertex3f((x+1)/8,y/8,-2+sqrt((float)((x+1)*(x+1)+(y+0)*(y+0)))/4.0);
			glTexCoord2f( (x+9)/16, (y+9)/16);
			glVertex3f((x+1)/8,(y+1)/8,-2+sqrt((float)((x+1)*(x+1)+(y+1)*(y+1)))/4.0);
			glTexCoord2f( (x+8)/16, (y+9)/16);
			glVertex3f(x/8,(y+1)/8,-2+sqrt((float)((x+0)*(x+0)+(y+1)*(y+1)))/4.0);
		}
	}
	glEnd();
	glDisable(GL_TEXTURE_2D);
	glPopMatrix();	
}

void DrawAxis()
{
   glBegin(GL_LINES);
   glColor3f(1,0,0);
   glVertex3f(0,0,0);
   glVertex3f(1,0,0);
   glColor3f(0,1,0);
   glVertex3f(0,0,0);
   glVertex3f(0,1,0);
   glColor3f(0,0,1);
   glVertex3f(0,0,0);
   glVertex3f(0,0,1);
   glEnd();	
}

void DrawPipe()
{
   gluCylinder(quadObj,0.075,0.075,1,16,1);
   glTranslatef(0,0,-0.25);
   gluCylinder(quadObj,0.1,0.1,0.25,16,1);	
   glTranslatef(0,0,0.25);
}

void DrawCube(float x,float y,float z)
{
	glScalef(x,y,z);
	glutSolidCube(1);
	glScalef(1/x,1/y,1/z);
}

void UpdateLasers()
{
	int i;
	
	for(i=0;i<MAX_NR_SHOTS;i++)
	{
		if(shots[i].active==TRUE)
		{
			DrawLaser(i);
			shots[i].px+=10*deltaTime;			
			if(shots[i].px>20) 
			{
				shots[i].active=FALSE;
				nrShots--;
			}
		}
	}
}

void SpawnLaser(float xp,float yp,float zp,float xr,float yr,float zr)
{
	int i,nr;
	
	if(nrShots<MAX_NR_SHOTS)
	{
		nrShots++;
		for(i=0;i<MAX_NR_SHOTS;i++)
		{
			if(shots[i].active==FALSE) nr=i;
		}
		shots[nr].px=xp;
		shots[nr].py=yp;
		shots[nr].pz=zp;
		shots[nr].rx=xr;
		shots[nr].ry=yr;
		shots[nr].rz=zr;
		shots[nr].active=TRUE;
	}
}
void DrawScene() 
{ 	
	static GLfloat rotX = 0.0;
   static GLfloat rotY = 0.0;
   static GLfloat rotZ = 0.0;
   static float lpos=0;
   static float lpos2=2;
   static float lasr1=0;
   static float lasr2=0;
   static float lasr3=0;
   static float lasr21=0;
   static float lasr22=0;
   static float lasr23=0;

   UpdateTimer();
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 

   glPushMatrix(); 
	if(!haltMovement)
	{
   	rotY = 360+270*sin(time*0.16)+30*sin(time*0.5);
   	rotZ = 25+15*sin(time*0.6)+10*sin(time*0.35);   	
   }
	DrawSpace();
   glTranslatef(0,-0.75,-1.25);
   glPushMatrix();
   lpos+=10*deltaTime;
   if (lpos>4) 
   {
   	lpos-=4;
   	SpawnLaser(0.75,0,0.25,0,rotY,rotZ);
   	#ifdef WIN
   	PlaySound("laser.wav", NULL, SND_ASYNC); 
   	#endif
   }
   lpos2+=10*deltaTime;
   if (lpos2>4) 
   {
   	lpos2-=4;
   	SpawnLaser(0.75,0,-0.25,0,rotY,rotZ);
   	#ifdef WIN
   	PlaySound("laser.wav", NULL, SND_ASYNC); 
   	#endif
   }
   glEnable(GL_DEPTH_TEST);
	glEnable(GL_LIGHT0);
   glEnable(GL_LIGHTING);
   glRotatef(0, 1.0, 0.0, 0.0);
   glRotatef(rotY, 0.0, 1.0, 0.0);
   glTranslatef(0.1,0,0);
   glRotatef(90,1,0,0);
   gluCylinder(quadObj,0.125,0.125,1.5,16,1);	
   glRotatef(-90,1,0,0);
   glTranslatef(-0.1,0,0);
   glRotatef(rotZ, 0.0, 0.0, 1.0);
   DrawCube(0.6,0.3,0.75);
   DrawCube(0.45,0.4,0.6);
   DrawCube(0.7,0.2,0.3);
   DrawCube(0.45,0.2,0.85);
   glRotatef(-90,0,1,0);
   glTranslatef(0.25,0,-0.75-lpos/20);
   DrawPipe();
   glTranslatef(-0.5,0,lpos/20-lpos2/20);
   DrawPipe();
   glDisable(GL_LIGHTING);
	glDisable(GL_LIGHT0);
   glPopMatrix(); 
   UpdateLasers();
	glDisable(GL_DEPTH_TEST);
   glPopMatrix();
   glutSwapBuffers(); 
}

void main(int argc, char **argv) 
{ 
   GLsizei iWidth = 640.0; 
   GLsizei iHeight = 480.0; 

   glutInit(&argc, argv);
   glutInitWindowPosition( 0, 0);
   glutInitWindowSize(iWidth, iHeight);
   glutInitDisplayMode( GLUT_RGB | GLUT_DEPTH | GLUT_DOUBLE );
   glutCreateWindow( "OpenGL Program by Andreas Gustafsson" );
   glutDisplayFunc(DrawScene );
   glutIdleFunc(Idle );
   glutReshapeFunc(Resize );
   glutKeyboardFunc(Keyboard );

	Init();
   glClearColor( 0.0, 0.0, 0.0, 0.0 ); 
   glEnable(GL_DEPTH_TEST); 
   glEnable(GL_NORMALIZE);

	#ifdef WIN
   QueryPerformanceFrequency(&TimeFreq);
   QueryPerformanceCounter(&TimeStart);
   #endif

   Resize(iWidth, iHeight); 
   glutMainLoop(); 
}
